# spatstat.utils NAMESPACE file

import(stats,graphics,grDevices,utils,methods)

# Do not edit the following.
# It is generated automatically.



# .................................................. 
#  load dynamic library
#      (native routines are now registered in init.c)
#      (entry points are symbols)
# .................................................. 

useDynLib(spatstat.utils, .registration=TRUE, .fixes="C_")

# .................................................. 
# Automatically-generated list of documented objects 
# .................................................. 
export("adjustthinrange")
export("apply23sum")
export("Area.xypolygon")       
export("articlebeforenumber")
export("as2vector")
export("asNumericMatrix")
export("assignDFcolumn")
export("badprobability")
export("bdrylength.xypolygon") 
export("blockdiagarray")
export("blockdiagmatrix")
export("can.be.formula")
export("capitalise")
export("cat.factor")
export("check.1.integer")
export("check.1.real")
export("check.1.string")
export("check.anyvector")
export("check.finite")
export("check.in.range")
export("check.named.list")
export("check.named.thing")
export("check.named.vector")
export("check.nmatrix")
export("check.nvector")
export("check.range")
export("check.satisfies")
export("choptext")
export("choptextline")
export("commasep")
export("complaining")
export("difflong")
export("distpl")               
export("distppl")
export("distppll")
export("distppllmin")
export("divisors")
export("do.call.matched")
export("do.call.without")
export("dont.complain.about")
export("dotexpr.to.call")
export("dropifsingle")
export("dround")
export("ensure2vector")
export("ensure3Darray")
export("equispaced")          
export("eratosthenes")
export("evenly.spaced")
export("exactCutBreaks")
export("exceedsMaxArraySize")
export("exhibitStringList")
export("expand.polynom")
export("explain.ifnot")
export("fakecallstring")
export("fastFindInterval")
export("fave.order")
export("fillseq")
export("findfirstfactor")
export("firstfactor")
export("flat.deparse")
export("fontify")
export("forbidNA")
export("fullIndexSequence")
export("geomseq")
export("getdataobjects")
export("getSpatstatLocatorQueue")
export("good.names")
export("graphicsPars")            
export("greatest.common.divisor")
export("grokIndexVector")
export("gsubdot")
export("harmonicmean")
export("harmonicsum")
export("identical.formulae")
export("ifelse0NA")
export("ifelse1NA")
export("ifelseAB")
export("ifelseAX")
export("ifelseNegPos")
export("ifelseXB")
export("ifelseXY")
export("indexCartesian")
export("inject.expr")
export("insertinlist")     
export("inside.range")
export("inside.triangle")
export("inside.xypolygon")     
export("intersect.ranges")
export("is.blank")
export("is.cube")
export("is.hole.xypolygon")
export("is.parseable")
export("is.power")
export("is.prime")
export("isRelevantZero")
export("is.square")
export("least.common.multiple")
export("lhs.of.formula")
export("lhs.of.formula<-")
export("logicalIndex")
export("lty2char")
export("makeCutLabels")
export("make.parseable")
export("mapstrings")
export("matchIntegerDataFrames")
export("matchNameOrPosition")
export("matcolall")
export("matcolany")
export("matcolsum")
export("matrixsample")
export("matrowall")
export("matrowany")
export("matrowsum")
export("natozero")
export("niceround")
export("NNdist2segments")
export("numalign")
export("nzpaste")
export("offsetsinformula")
export("optimizeWithTrace")
export("orderstats")
export("orderwhich")
export("ordinal")
export("ordinalsuffix")
export("overlap.trapezium")
export("overlap.xypolygon")    
export("padtowidth")
export("paren")
export("passthrough")
export("paste.expr")
export("pasteFormula")
export("pasteN")
export("percentage")
export("positiveIndex")
export("prange")
export("prettydiscrete")
export("prettyinside")
export("primefactors")
export("primesbelow")
export("prolongseq")
export("putSpatstatLocatorQueue")
export("queueSpatstatLocator")
export("ratiotweak")
export("relatively.prime")
export("RelevantEmpty")
export("RelevantNA")
export("RelevantZero")
export("replacementIndex")
export("resolve.1.default")
export("resolve.defaults")
export("revcumsum")
export("reverse.xypolygon")    
export("rhs.of.formula")
export("rhs.of.formula<-")
export("samefunction")
export("sensiblevarname")
export("short.deparse")
export("sideCode")
export("simplenumber")
export("simplify.xypolygon")
export("singlestring")
export("snapxy")
export("sortunique")
export("spatstatLocator")
export("splat")
export("startinrange")
export("strictIndexSequence")
export("strsplitretain")
export("substringcount")
export("sympoly")
export("taperoff")
export("tapplysum")
export("termsinformula")
export("there.can.be.only.one")
export("there.is.no.try")
export("trap.extra.arguments")
export("truncline")
export("unparen")
export("uptrimat")
export("validposint")
export("variablesinformula")
export("variablesintext")
export("verbalogic")
export("verify.xypolygon")
export("warn.ignored.args")
export("warn.reset.arg")
export("which.max.fair")
export("which.min.fair")
export("xysegMcircle")
export("xysegPcircle")
export("xysegXcircle")

# .......  Special cases ........... 
export("%orifnull%")
# .......  End of special cases  ... 

# ......................................... 
# Automatically generated list of S3 methods
# ......................................... 
# ......................................... 
#      Assignment methods                   
# ......................................... 
# ......................................... 
#      End of methods                       
# ......................................... 
